@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -169,21 +169,23 @@ keyserver_any_configured (ctrl_t ctrl)
 }
 
 int
-keyserver_import_keyid (u32 *keyid, void *dummy)
+keyserver_import_keyid (u32 *keyid, void *dummy, int quick)
 {
   (void)keyid;
   (void)dummy;
+  (void)quick;
   return -1;
 }
 
 int
 keyserver_import_fprint (ctrl_t ctrl, const byte *fprint,size_t fprint_len,
-			 struct keyserver_spec *keyserver)
+			 struct keyserver_spec *keyserver, int quick)
 {
   (void)ctrl;
   (void)fprint;
   (void)fprint_len;
   (void)keyserver;
+  (void)quick;
   return -1;
 }
 
@@ -203,11 +205,12 @@ keyserver_import_pka (const char *name,unsigned char *fpr)
 }
 
 gpg_error_t
-keyserver_import_wkd (ctrl_t ctrl, const char *name,
+keyserver_import_wkd (ctrl_t ctrl, const char *name, int quick,
                       unsigned char **fpr, size_t *fpr_len)
 {
   (void)ctrl;
   (void)name;
+  (void)quick;
   (void)fpr;
   (void)fpr_len;
   return GPG_ERR_BUG;
@@ -294,15 +297,13 @@ check_secret_key (PKT_public_key *pk, int n)
  * No secret key, so no passphrase needed
  */
 DEK *
-passphrase_to_dek (u32 *keyid, int pubkey_algo,
-                   int cipher_algo, STRING2KEY *s2k, int mode,
+passphrase_to_dek (int cipher_algo, STRING2KEY *s2k, int create, int nocache,
                    const char *tmp, int *canceled)
 {
-  (void)keyid;
-  (void)pubkey_algo;
   (void)cipher_algo;
   (void)s2k;
-  (void)mode;
+  (void)create;
+  (void)nocache;
   (void)tmp;
 
   if (canceled)
@@ -311,11 +312,9 @@ passphrase_to_dek (u32 *keyid, int pubkey_algo,
 }
 
 void
-passphrase_clear_cache (u32 *keyid, const char *cacheid, int algo)
+passphrase_clear_cache (const char *cacheid)
 {
-  (void)keyid;
   (void)cacheid;
-  (void)algo;
 }
 
 struct keyserver_spec *
@@ -477,6 +476,17 @@ export_pubkey_buffer (ctrl_t ctrl, const char *keyspec, unsigned int options,
   return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 }
 
+gpg_error_t
+tofu_write_tfs_record (ctrl_t ctrl, estream_t fp,
+                       PKT_public_key *pk, const char *user_id)
+{
+  (void)ctrl;
+  (void)fp;
+  (void)pk;
+  (void)user_id;
+  return gpg_error (GPG_ERR_GENERAL);
+}
+
 gpg_error_t
 tofu_get_policy (ctrl_t ctrl, PKT_public_key *pk, PKT_user_id *user_id,
 		 enum tofu_policy *policy)
@@ -497,11 +507,13 @@ tofu_policy_str (enum tofu_policy policy)
 }
 
 void
-tofu_begin_batch_update (void)
+tofu_begin_batch_update (ctrl_t ctrl)
 {
+  (void)ctrl;
 }
 
 void
-tofu_end_batch_update (void)
+tofu_end_batch_update (ctrl_t ctrl)
 {
+  (void)ctrl;
 }
